(* Copyright (C) 2013 Benjamin Monmege 
   LSV, ENS Cachan & CNRS
   <benjamin.monmege@lsv.ens-cachan.fr>

   This program is free software: you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see
   <http://www.gnu.org/licenses/>. *)

open Modules
open Formula	
open Expression


let read_args() = (* returns the expression, the word, and a flag for the timer *)
  let e = ref "" and u = ref "" and timer = ref false in
  let speclist = 
    [("-expr",Arg.String (fun s -> e:=s),"Hybrid weighted expression to be evaluated");
     ("-word",Arg.String (fun s -> u:=s),"Word on which the expression should be evaluated");
     ("-timer",Arg.Unit (fun () -> timer:=true), "Use a timer")]
  in let usage_msg = "Usage: QuantiS -expr <expression> -word <non-empty string> [-timer]" in 
     Arg.parse speclist (fun s -> ()) usage_msg;
     (!e,!u,!timer);;

let _ = 
  let (e,u,timer)=read_args() in

  print_newline();
  print_string "Evaluation of expression: "; print_newline();
  print_string ("   "^e) ; print_newline ();
  print_string "on string: "; print_newline ();
  print_string ("   "^u); print_newline ();
  print_string "Result: ";
  let expr = Expr.input Lexer.token (Lexing.from_string e) in
  if timer then
    let t = Sys.time() in
    let result = Kleene.eval_expression expr u in
    let total_time = Sys.time()-.t in
    print_float result; print_newline ();
    print_string ("Time: "); print_float total_time; print_string("s");
    print_newline (); print_newline()
  else 
    let result = Kleene.eval_expression expr u in
    print_float result; 
    print_newline (); print_newline();;

let parse_error s = (* Called by the parser function on error *)
  print_endline s;
  flush stdout
